# pylint: disable=too-many-lines
# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
from io import IOBase
from typing import Any, AsyncIterable, Callable, Dict, IO, Optional, TypeVar, Union, cast, overload
import urllib.parse

from azure.core.async_paging import AsyncItemPaged, AsyncList
from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    map_error,
)
from azure.core.pipeline import PipelineResponse
from azure.core.pipeline.transport import AsyncHttpResponse
from azure.core.polling import AsyncLROPoller, AsyncNoPolling, AsyncPollingMethod
from azure.core.rest import HttpRequest
from azure.core.tracing.decorator import distributed_trace
from azure.core.tracing.decorator_async import distributed_trace_async
from azure.core.utils import case_insensitive_dict
from azure.mgmt.core.exceptions import ARMErrorFormat
from azure.mgmt.core.polling.async_arm_polling import AsyncARMPolling

from ... import models as _models
from ..._vendor import _convert_request
from ...operations._communication_services_operations import (
    build_check_name_availability_request,
    build_create_or_update_request,
    build_delete_request,
    build_get_request,
    build_link_notification_hub_request,
    build_list_by_resource_group_request,
    build_list_by_subscription_request,
    build_list_keys_request,
    build_regenerate_key_request,
    build_update_request,
)

T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, AsyncHttpResponse], T, Dict[str, Any]], Any]]


class CommunicationServicesOperations:
    """
    .. warning::
        **DO NOT** instantiate this class directly.

        Instead, you should access the following operations through
        :class:`~azure.mgmt.communication.aio.CommunicationServiceManagementClient`'s
        :attr:`communication_services` attribute.
    """

    models = _models

    def __init__(self, *args, **kwargs) -> None:
        input_args = list(args)
        self._client = input_args.pop(0) if input_args else kwargs.pop("client")
        self._config = input_args.pop(0) if input_args else kwargs.pop("config")
        self._serialize = input_args.pop(0) if input_args else kwargs.pop("serializer")
        self._deserialize = input_args.pop(0) if input_args else kwargs.pop("deserializer")

    @overload
    async def check_name_availability(
        self,
        name_availability_parameters: _models.NameAvailabilityParameters,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.CheckNameAvailabilityResponse:
        """Check Name Availability.

        Checks that the CommunicationService name is valid and is not already in use.

        :param name_availability_parameters: Parameters supplied to the operation. Required.
        :type name_availability_parameters: ~azure.mgmt.communication.models.NameAvailabilityParameters
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CheckNameAvailabilityResponse or the result of cls(response)
        :rtype: ~azure.mgmt.communication.models.CheckNameAvailabilityResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def check_name_availability(
        self, name_availability_parameters: IO, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.CheckNameAvailabilityResponse:
        """Check Name Availability.

        Checks that the CommunicationService name is valid and is not already in use.

        :param name_availability_parameters: Parameters supplied to the operation. Required.
        :type name_availability_parameters: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CheckNameAvailabilityResponse or the result of cls(response)
        :rtype: ~azure.mgmt.communication.models.CheckNameAvailabilityResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def check_name_availability(
        self, name_availability_parameters: Union[_models.NameAvailabilityParameters, IO], **kwargs: Any
    ) -> _models.CheckNameAvailabilityResponse:
        """Check Name Availability.

        Checks that the CommunicationService name is valid and is not already in use.

        :param name_availability_parameters: Parameters supplied to the operation. Is either a
         NameAvailabilityParameters type or a IO type. Required.
        :type name_availability_parameters: ~azure.mgmt.communication.models.NameAvailabilityParameters
         or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CheckNameAvailabilityResponse or the result of cls(response)
        :rtype: ~azure.mgmt.communication.models.CheckNameAvailabilityResponse
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.CheckNameAvailabilityResponse] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(name_availability_parameters, (IOBase, bytes)):
            _content = name_availability_parameters
        else:
            _json = self._serialize.body(name_availability_parameters, "NameAvailabilityParameters")

        request = build_check_name_availability_request(
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.check_name_availability.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("CheckNameAvailabilityResponse", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    check_name_availability.metadata = {
        "url": "/subscriptions/{subscriptionId}/providers/Microsoft.Communication/checkNameAvailability"
    }

    @overload
    async def link_notification_hub(
        self,
        resource_group_name: str,
        communication_service_name: str,
        link_notification_hub_parameters: Optional[_models.LinkNotificationHubParameters] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.LinkedNotificationHub:
        """Link Notification Hub.

        Links an Azure Notification Hub to this communication service.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :param link_notification_hub_parameters: Parameters supplied to the operation. Default value is
         None.
        :type link_notification_hub_parameters:
         ~azure.mgmt.communication.models.LinkNotificationHubParameters
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: LinkedNotificationHub or the result of cls(response)
        :rtype: ~azure.mgmt.communication.models.LinkedNotificationHub
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def link_notification_hub(
        self,
        resource_group_name: str,
        communication_service_name: str,
        link_notification_hub_parameters: Optional[IO] = None,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.LinkedNotificationHub:
        """Link Notification Hub.

        Links an Azure Notification Hub to this communication service.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :param link_notification_hub_parameters: Parameters supplied to the operation. Default value is
         None.
        :type link_notification_hub_parameters: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: LinkedNotificationHub or the result of cls(response)
        :rtype: ~azure.mgmt.communication.models.LinkedNotificationHub
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def link_notification_hub(
        self,
        resource_group_name: str,
        communication_service_name: str,
        link_notification_hub_parameters: Optional[Union[_models.LinkNotificationHubParameters, IO]] = None,
        **kwargs: Any
    ) -> _models.LinkedNotificationHub:
        """Link Notification Hub.

        Links an Azure Notification Hub to this communication service.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :param link_notification_hub_parameters: Parameters supplied to the operation. Is either a
         LinkNotificationHubParameters type or a IO type. Default value is None.
        :type link_notification_hub_parameters:
         ~azure.mgmt.communication.models.LinkNotificationHubParameters or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: LinkedNotificationHub or the result of cls(response)
        :rtype: ~azure.mgmt.communication.models.LinkedNotificationHub
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.LinkedNotificationHub] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(link_notification_hub_parameters, (IOBase, bytes)):
            _content = link_notification_hub_parameters
        else:
            if link_notification_hub_parameters is not None:
                _json = self._serialize.body(link_notification_hub_parameters, "LinkNotificationHubParameters")
            else:
                _json = None

        request = build_link_notification_hub_request(
            resource_group_name=resource_group_name,
            communication_service_name=communication_service_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.link_notification_hub.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("LinkedNotificationHub", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    link_notification_hub.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Communication/communicationServices/{communicationServiceName}/linkNotificationHub"
    }

    @distributed_trace
    def list_by_subscription(self, **kwargs: Any) -> AsyncIterable["_models.CommunicationServiceResource"]:
        """List By Subscription.

        Handles requests to list all resources in a subscription.

        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either CommunicationServiceResource or the result of
         cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.communication.models.CommunicationServiceResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.CommunicationServiceResourceList] = kwargs.pop("cls", None)

        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                request = build_list_by_subscription_request(
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    template_url=self.list_by_subscription.metadata["url"],
                    headers=_headers,
                    params=_params,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("CommunicationServiceResourceList", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    list_by_subscription.metadata = {
        "url": "/subscriptions/{subscriptionId}/providers/Microsoft.Communication/communicationServices"
    }

    @distributed_trace
    def list_by_resource_group(
        self, resource_group_name: str, **kwargs: Any
    ) -> AsyncIterable["_models.CommunicationServiceResource"]:
        """List By Resource Group.

        Handles requests to list all resources in a resource group.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: An iterator like instance of either CommunicationServiceResource or the result of
         cls(response)
        :rtype:
         ~azure.core.async_paging.AsyncItemPaged[~azure.mgmt.communication.models.CommunicationServiceResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.CommunicationServiceResourceList] = kwargs.pop("cls", None)

        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                request = build_list_by_resource_group_request(
                    resource_group_name=resource_group_name,
                    subscription_id=self._config.subscription_id,
                    api_version=api_version,
                    template_url=self.list_by_resource_group.metadata["url"],
                    headers=_headers,
                    params=_params,
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                request = _convert_request(request)
                request.url = self._client.format_url(request.url)
                request.method = "GET"
            return request

        async def extract_data(pipeline_response):
            deserialized = self._deserialize("CommunicationServiceResourceList", pipeline_response)
            list_of_elem = deserialized.value
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.next_link or None, AsyncList(list_of_elem)

        async def get_next(next_link=None):
            request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
                request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
                raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

            return pipeline_response

        return AsyncItemPaged(get_next, extract_data)

    list_by_resource_group.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Communication/communicationServices"
    }

    @overload
    async def update(
        self,
        resource_group_name: str,
        communication_service_name: str,
        parameters: _models.CommunicationServiceResourceUpdate,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.CommunicationServiceResource:
        """Update.

        Operation to update an existing CommunicationService.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :param parameters: Parameters for the update operation. Required.
        :type parameters: ~azure.mgmt.communication.models.CommunicationServiceResourceUpdate
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CommunicationServiceResource or the result of cls(response)
        :rtype: ~azure.mgmt.communication.models.CommunicationServiceResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def update(
        self,
        resource_group_name: str,
        communication_service_name: str,
        parameters: IO,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.CommunicationServiceResource:
        """Update.

        Operation to update an existing CommunicationService.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :param parameters: Parameters for the update operation. Required.
        :type parameters: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CommunicationServiceResource or the result of cls(response)
        :rtype: ~azure.mgmt.communication.models.CommunicationServiceResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def update(
        self,
        resource_group_name: str,
        communication_service_name: str,
        parameters: Union[_models.CommunicationServiceResourceUpdate, IO],
        **kwargs: Any
    ) -> _models.CommunicationServiceResource:
        """Update.

        Operation to update an existing CommunicationService.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :param parameters: Parameters for the update operation. Is either a
         CommunicationServiceResourceUpdate type or a IO type. Required.
        :type parameters: ~azure.mgmt.communication.models.CommunicationServiceResourceUpdate or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CommunicationServiceResource or the result of cls(response)
        :rtype: ~azure.mgmt.communication.models.CommunicationServiceResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.CommunicationServiceResource] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "CommunicationServiceResourceUpdate")

        request = build_update_request(
            resource_group_name=resource_group_name,
            communication_service_name=communication_service_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.update.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("CommunicationServiceResource", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    update.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Communication/communicationServices/{communicationServiceName}"
    }

    @distributed_trace_async
    async def get(
        self, resource_group_name: str, communication_service_name: str, **kwargs: Any
    ) -> _models.CommunicationServiceResource:
        """Get.

        Get the CommunicationService and its properties.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CommunicationServiceResource or the result of cls(response)
        :rtype: ~azure.mgmt.communication.models.CommunicationServiceResource
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.CommunicationServiceResource] = kwargs.pop("cls", None)

        request = build_get_request(
            resource_group_name=resource_group_name,
            communication_service_name=communication_service_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self.get.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("CommunicationServiceResource", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    get.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Communication/communicationServices/{communicationServiceName}"
    }

    async def _create_or_update_initial(
        self,
        resource_group_name: str,
        communication_service_name: str,
        parameters: Union[_models.CommunicationServiceResource, IO],
        **kwargs: Any
    ) -> _models.CommunicationServiceResource:
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.CommunicationServiceResource] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "CommunicationServiceResource")

        request = build_create_or_update_request(
            resource_group_name=resource_group_name,
            communication_service_name=communication_service_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self._create_or_update_initial.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 201]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        response_headers = {}
        if response.status_code == 200:
            deserialized = self._deserialize("CommunicationServiceResource", pipeline_response)

        if response.status_code == 201:
            response_headers["Azure-AsyncOperation"] = self._deserialize(
                "str", response.headers.get("Azure-AsyncOperation")
            )

            deserialized = self._deserialize("CommunicationServiceResource", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore

    _create_or_update_initial.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Communication/communicationServices/{communicationServiceName}"
    }

    @overload
    async def begin_create_or_update(
        self,
        resource_group_name: str,
        communication_service_name: str,
        parameters: _models.CommunicationServiceResource,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncLROPoller[_models.CommunicationServiceResource]:
        """Create Or Update.

        Create a new CommunicationService or update an existing CommunicationService.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :param parameters: Parameters for the create or update operation. Required.
        :type parameters: ~azure.mgmt.communication.models.CommunicationServiceResource
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be AsyncARMPolling. Pass in False for
         this operation to not poll, or pass in your own initialized polling object for a personal
         polling strategy.
        :paramtype polling: bool or ~azure.core.polling.AsyncPollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of AsyncLROPoller that returns either CommunicationServiceResource or the
         result of cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.communication.models.CommunicationServiceResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def begin_create_or_update(
        self,
        resource_group_name: str,
        communication_service_name: str,
        parameters: IO,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> AsyncLROPoller[_models.CommunicationServiceResource]:
        """Create Or Update.

        Create a new CommunicationService or update an existing CommunicationService.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :param parameters: Parameters for the create or update operation. Required.
        :type parameters: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be AsyncARMPolling. Pass in False for
         this operation to not poll, or pass in your own initialized polling object for a personal
         polling strategy.
        :paramtype polling: bool or ~azure.core.polling.AsyncPollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of AsyncLROPoller that returns either CommunicationServiceResource or the
         result of cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.communication.models.CommunicationServiceResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def begin_create_or_update(
        self,
        resource_group_name: str,
        communication_service_name: str,
        parameters: Union[_models.CommunicationServiceResource, IO],
        **kwargs: Any
    ) -> AsyncLROPoller[_models.CommunicationServiceResource]:
        """Create Or Update.

        Create a new CommunicationService or update an existing CommunicationService.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :param parameters: Parameters for the create or update operation. Is either a
         CommunicationServiceResource type or a IO type. Required.
        :type parameters: ~azure.mgmt.communication.models.CommunicationServiceResource or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be AsyncARMPolling. Pass in False for
         this operation to not poll, or pass in your own initialized polling object for a personal
         polling strategy.
        :paramtype polling: bool or ~azure.core.polling.AsyncPollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of AsyncLROPoller that returns either CommunicationServiceResource or the
         result of cls(response)
        :rtype:
         ~azure.core.polling.AsyncLROPoller[~azure.mgmt.communication.models.CommunicationServiceResource]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.CommunicationServiceResource] = kwargs.pop("cls", None)
        polling: Union[bool, AsyncPollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = await self._create_or_update_initial(
                resource_group_name=resource_group_name,
                communication_service_name=communication_service_name,
                parameters=parameters,
                api_version=api_version,
                content_type=content_type,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):
            deserialized = self._deserialize("CommunicationServiceResource", pipeline_response)
            if cls:
                return cls(pipeline_response, deserialized, {})
            return deserialized

        if polling is True:
            polling_method: AsyncPollingMethod = cast(
                AsyncPollingMethod,
                AsyncARMPolling(lro_delay, lro_options={"final-state-via": "azure-async-operation"}, **kwargs),
            )
        elif polling is False:
            polling_method = cast(AsyncPollingMethod, AsyncNoPolling())
        else:
            polling_method = polling
        if cont_token:
            return AsyncLROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return AsyncLROPoller(self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    begin_create_or_update.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Communication/communicationServices/{communicationServiceName}"
    }

    async def _delete_initial(  # pylint: disable=inconsistent-return-statements
        self, resource_group_name: str, communication_service_name: str, **kwargs: Any
    ) -> None:
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)

        request = build_delete_request(
            resource_group_name=resource_group_name,
            communication_service_name=communication_service_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self._delete_initial.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200, 202, 204]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        response_headers = {}
        if response.status_code == 202:
            response_headers["location"] = self._deserialize("str", response.headers.get("location"))

        if cls:
            return cls(pipeline_response, None, response_headers)

    _delete_initial.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Communication/communicationServices/{communicationServiceName}"
    }

    @distributed_trace_async
    async def begin_delete(
        self, resource_group_name: str, communication_service_name: str, **kwargs: Any
    ) -> AsyncLROPoller[None]:
        """Delete.

        Operation to delete a CommunicationService.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :keyword str continuation_token: A continuation token to restart a poller from a saved state.
        :keyword polling: By default, your polling method will be AsyncARMPolling. Pass in False for
         this operation to not poll, or pass in your own initialized polling object for a personal
         polling strategy.
        :paramtype polling: bool or ~azure.core.polling.AsyncPollingMethod
        :keyword int polling_interval: Default waiting time between two polls for LRO operations if no
         Retry-After header is present.
        :return: An instance of AsyncLROPoller that returns either None or the result of cls(response)
        :rtype: ~azure.core.polling.AsyncLROPoller[None]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[None] = kwargs.pop("cls", None)
        polling: Union[bool, AsyncPollingMethod] = kwargs.pop("polling", True)
        lro_delay = kwargs.pop("polling_interval", self._config.polling_interval)
        cont_token: Optional[str] = kwargs.pop("continuation_token", None)
        if cont_token is None:
            raw_result = await self._delete_initial(  # type: ignore
                resource_group_name=resource_group_name,
                communication_service_name=communication_service_name,
                api_version=api_version,
                cls=lambda x, y, z: x,
                headers=_headers,
                params=_params,
                **kwargs
            )
        kwargs.pop("error_map", None)

        def get_long_running_output(pipeline_response):  # pylint: disable=inconsistent-return-statements
            if cls:
                return cls(pipeline_response, None, {})

        if polling is True:
            polling_method: AsyncPollingMethod = cast(
                AsyncPollingMethod, AsyncARMPolling(lro_delay, lro_options={"final-state-via": "location"}, **kwargs)
            )
        elif polling is False:
            polling_method = cast(AsyncPollingMethod, AsyncNoPolling())
        else:
            polling_method = polling
        if cont_token:
            return AsyncLROPoller.from_continuation_token(
                polling_method=polling_method,
                continuation_token=cont_token,
                client=self._client,
                deserialization_callback=get_long_running_output,
            )
        return AsyncLROPoller(self._client, raw_result, get_long_running_output, polling_method)  # type: ignore

    begin_delete.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Communication/communicationServices/{communicationServiceName}"
    }

    @distributed_trace_async
    async def list_keys(
        self, resource_group_name: str, communication_service_name: str, **kwargs: Any
    ) -> _models.CommunicationServiceKeys:
        """List Keys.

        Get the access keys of the CommunicationService resource.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CommunicationServiceKeys or the result of cls(response)
        :rtype: ~azure.mgmt.communication.models.CommunicationServiceKeys
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        cls: ClsType[_models.CommunicationServiceKeys] = kwargs.pop("cls", None)

        request = build_list_keys_request(
            resource_group_name=resource_group_name,
            communication_service_name=communication_service_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            template_url=self.list_keys.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("CommunicationServiceKeys", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    list_keys.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Communication/communicationServices/{communicationServiceName}/listKeys"
    }

    @overload
    async def regenerate_key(
        self,
        resource_group_name: str,
        communication_service_name: str,
        parameters: _models.RegenerateKeyParameters,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.CommunicationServiceKeys:
        """Regenerate Key.

        Regenerate CommunicationService access key. PrimaryKey and SecondaryKey cannot be regenerated
        at the same time.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :param parameters: Parameter that describes the Regenerate Key Operation. Required.
        :type parameters: ~azure.mgmt.communication.models.RegenerateKeyParameters
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CommunicationServiceKeys or the result of cls(response)
        :rtype: ~azure.mgmt.communication.models.CommunicationServiceKeys
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    async def regenerate_key(
        self,
        resource_group_name: str,
        communication_service_name: str,
        parameters: IO,
        *,
        content_type: str = "application/json",
        **kwargs: Any
    ) -> _models.CommunicationServiceKeys:
        """Regenerate Key.

        Regenerate CommunicationService access key. PrimaryKey and SecondaryKey cannot be regenerated
        at the same time.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :param parameters: Parameter that describes the Regenerate Key Operation. Required.
        :type parameters: IO
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CommunicationServiceKeys or the result of cls(response)
        :rtype: ~azure.mgmt.communication.models.CommunicationServiceKeys
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace_async
    async def regenerate_key(
        self,
        resource_group_name: str,
        communication_service_name: str,
        parameters: Union[_models.RegenerateKeyParameters, IO],
        **kwargs: Any
    ) -> _models.CommunicationServiceKeys:
        """Regenerate Key.

        Regenerate CommunicationService access key. PrimaryKey and SecondaryKey cannot be regenerated
        at the same time.

        :param resource_group_name: The name of the resource group. The name is case insensitive.
         Required.
        :type resource_group_name: str
        :param communication_service_name: The name of the CommunicationService resource. Required.
        :type communication_service_name: str
        :param parameters: Parameter that describes the Regenerate Key Operation. Is either a
         RegenerateKeyParameters type or a IO type. Required.
        :type parameters: ~azure.mgmt.communication.models.RegenerateKeyParameters or IO
        :keyword content_type: Body Parameter content-type. Known values are: 'application/json'.
         Default value is None.
        :paramtype content_type: str
        :keyword callable cls: A custom type or function that will be passed the direct response
        :return: CommunicationServiceKeys or the result of cls(response)
        :rtype: ~azure.mgmt.communication.models.CommunicationServiceKeys
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

        api_version: str = kwargs.pop("api_version", _params.pop("api-version", self._config.api_version))
        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.CommunicationServiceKeys] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _json = None
        _content = None
        if isinstance(parameters, (IOBase, bytes)):
            _content = parameters
        else:
            _json = self._serialize.body(parameters, "RegenerateKeyParameters")

        request = build_regenerate_key_request(
            resource_group_name=resource_group_name,
            communication_service_name=communication_service_name,
            subscription_id=self._config.subscription_id,
            api_version=api_version,
            content_type=content_type,
            json=_json,
            content=_content,
            template_url=self.regenerate_key.metadata["url"],
            headers=_headers,
            params=_params,
        )
        request = _convert_request(request)
        request.url = self._client.format_url(request.url)

        _stream = False
        pipeline_response: PipelineResponse = await self._client._pipeline.run(  # pylint: disable=protected-access
            request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            error = self._deserialize.failsafe_deserialize(_models.ErrorResponse, pipeline_response)
            raise HttpResponseError(response=response, model=error, error_format=ARMErrorFormat)

        deserialized = self._deserialize("CommunicationServiceKeys", pipeline_response)

        if cls:
            return cls(pipeline_response, deserialized, {})

        return deserialized

    regenerate_key.metadata = {
        "url": "/subscriptions/{subscriptionId}/resourceGroups/{resourceGroupName}/providers/Microsoft.Communication/communicationServices/{communicationServiceName}/regenerateKey"
    }



# coding=utf-8
# --------------------------------------------------------------------------
# Code generated by Microsoft (R) AutoRest Code Generator (autorest: 3.9.7, generator: @autorest/python@6.7.8)
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from copy import deepcopy
from typing import Any, Awaitable, TYPE_CHECKING

from azure.core import AsyncPipelineClient
from azure.core.rest import AsyncHttpResponse, HttpRequest

from .. import models as _models
from .._serialization import Deserializer, Serializer
from ._configuration import ContainerRegistryConfiguration
from .operations import AuthenticationOperations, ContainerRegistryBlobOperations, ContainerRegistryOperations

if TYPE_CHECKING:
    # pylint: disable=unused-import,ungrouped-imports
    from azure.core.credentials_async import AsyncTokenCredential


class ContainerRegistry:  # pylint: disable=client-accepts-api-version-keyword
    """Metadata API definition for the Azure Container Registry runtime.

    :ivar container_registry: ContainerRegistryOperations operations
    :vartype container_registry: container_registry.aio.operations.ContainerRegistryOperations
    :ivar container_registry_blob: ContainerRegistryBlobOperations operations
    :vartype container_registry_blob:
     container_registry.aio.operations.ContainerRegistryBlobOperations
    :ivar authentication: AuthenticationOperations operations
    :vartype authentication: container_registry.aio.operations.AuthenticationOperations
    :param url: Registry login URL. Required.
    :type url: str
    :param credential: Credential needed for the client to connect to Azure. Required.
    :type credential: ~azure.core.credentials_async.AsyncTokenCredential
    :keyword api_version: Api Version. Default value is "2021-07-01". Note that overriding this
     default value may result in unsupported behavior.
    :paramtype api_version: str
    """

    def __init__(self, url: str, credential: "AsyncTokenCredential", **kwargs: Any) -> None:
        _endpoint = "{url}"
        self._config = ContainerRegistryConfiguration(url=url, credential=credential, **kwargs)
        self._client: AsyncPipelineClient = AsyncPipelineClient(base_url=_endpoint, config=self._config, **kwargs)

        client_models = {k: v for k, v in _models._models.__dict__.items() if isinstance(v, type)}
        client_models.update({k: v for k, v in _models.__dict__.items() if isinstance(v, type)})
        self._serialize = Serializer(client_models)
        self._deserialize = Deserializer(client_models)
        self._serialize.client_side_validation = False
        self.container_registry = ContainerRegistryOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.container_registry_blob = ContainerRegistryBlobOperations(
            self._client, self._config, self._serialize, self._deserialize
        )
        self.authentication = AuthenticationOperations(self._client, self._config, self._serialize, self._deserialize)

    def send_request(self, request: HttpRequest, **kwargs: Any) -> Awaitable[AsyncHttpResponse]:
        """Runs the network request through the client's chained policies.

        >>> from azure.core.rest import HttpRequest
        >>> request = HttpRequest("GET", "https://www.example.org/")
        <HttpRequest [GET], url: 'https://www.example.org/'>
        >>> response = await client.send_request(request)
        <AsyncHttpResponse: 200 OK>

        For more information on this code flow, see https://aka.ms/azsdk/dpcodegen/python/send_request

        :param request: The network request you want to make. Required.
        :type request: ~azure.core.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~azure.core.rest.AsyncHttpResponse
        """

        request_copy = deepcopy(request)
        path_format_arguments = {
            "url": self._serialize.url("self._config.url", self._config.url, "str", skip_quote=True),
        }

        request_copy.url = self._client.format_url(request_copy.url, **path_format_arguments)
        return self._client.send_request(request_copy, **kwargs)

    async def close(self) -> None:
        await self._client.close()

    async def __aenter__(self) -> "ContainerRegistry":
        await self._client.__aenter__()
        return self

    async def __aexit__(self, *exc_details: Any) -> None:
        await self._client.__aexit__(*exc_details)



from datetime import date
from typing import Set, List, Dict, Any
import os
import logging
from github import Github
from github.Issue import Issue
from subprocess import check_call

_LOG = logging.getLogger(__name__)

_LANGUAGE_OWNER = {'msyyc'}
_LANGUAGE_REPO = 'Azure/azure-sdk-for-python'
_FILE_OUT_NAME = 'common.md'


class Common:
    """ The class defines some function for all languages to reference
    language_owner = {}  # language owner who may handle issue
    repo_name = ''  # base repo. For example: Azure/azure-sdk-for-python
    file_out_name = ''  # report name. For example: sdk_issue_status_python.md
    issue_html = ''  # issue url. For example: https://github.com/Azure/azure-sdk-for-python/issues

    """

    def __init__(self, language_owner: Set[str], repo_name: str, file_out_name: str):
        self.language_owner = language_owner
        self.repo_name = repo_name
        self.file_out_name = file_out_name
        self.issue_html = f'https://github.com/{repo_name}/issues'
        self.bot_assignees = {'msftbot[bot]'}

    @staticmethod
    def push_md_to_storage():
        cmd_list = ['git add .', 'git commit -m \"update excel\"', 'git push -f origin HEAD']
        [check_call(cmd, shell=True) for cmd in cmd_list]

    def collect_open_issues(self) -> List[Issue]:
        hub = Github(os.getenv('TOKEN'))
        repo = hub.get_repo(self.repo_name)
        mgmt_label = repo.get_label('Mgmt')
        open_issues = repo.get_issues(state='open', labels=[mgmt_label])
        return [issue for issue in open_issues if not issue.pull_request]

    def judge_status(self, issue: Issue) -> str:
        if issue.user.login in self.language_owner:
            return ''
        latest_comments = ''
        comments = [(comment.updated_at.timestamp(), comment.user.login) for comment in issue.get_comments()
                    if comment.user.login not in self.bot_assignees]
        comments.sort()
        if comments:
            latest_comments = comments[-1][1]
        if issue.comments == 0:
            return 'new issue'
        elif latest_comments not in self.language_owner:
            return 'new comment'
        else:
            return ''

    def extract_info(self, open_issues: List[Issue]) -> List[Dict[str, Any]]:
        issues_info = []
        _LOG.info(f'collect {len(open_issues)} open issues in {self.issue_html}')
        idx = 1
        for issue in open_issues:
            assignees = {assignee.login for assignee in issue.assignees}
            if assignees.intersection(self.language_owner):
                issue_info = {
                    'idx': idx,
                    'number': issue.number,
                    'html': f'{self.issue_html}/{issue.number}',
                    'title': issue.title,
                    'labels': [label.name for label in issue.labels],
                    'assignees': assignees,
                    'created_date': str(date.fromtimestamp(issue.created_at.timestamp()).strftime('%Y-%m-%d')),
                    'status': self.judge_status(issue)
                }
                issues_info.append(issue_info)
                idx = idx + 1
        _LOG.info(f'collect {len(issues_info)} open issues assigned to {str(self.language_owner)}')
        return issues_info

    @staticmethod
    def output_line(issue_info: Dict[str, Any]) -> str:
        return '|{No}|[#{number}]({issue_html})|{title}|{labels}|{assignees}|{bot_advice}|{created_date}|\n'.format(
            No=issue_info['idx'],
            number=issue_info['number'],
            issue_html=issue_info['html'],
            title=issue_info['title'],
            labels=', '.join(issue_info['labels']),
            assignees=', '.join(issue_info['assignees']),
            bot_advice=issue_info['status'],
            created_date=issue_info['created_date'],
        )

    def report(self, issues_info: List[Dict[str, Any]]) -> None:
        with open(self.file_out_name, 'w') as file_out:
            file_out.write(
                '| No. | issue | title | labels | assignees | bot advice | created date |\n')
            file_out.write('| ------ | ------ | ------ | ------ | ------ | ------ | :-----: |\n')
            file_out.writelines([Common.output_line(issue_info) for issue_info in issues_info])

    def run(self):
        open_issues = self.collect_open_issues()
        issues_info = self.extract_info(open_issues)
        self.report(issues_info)


def common_process():
    instance = Common(_LANGUAGE_OWNER, _LANGUAGE_REPO, _FILE_OUT_NAME)
    instance.run()



#!/usr/bin/env python

# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

# This script is intended to be executed from within a tox script. It takes care of of unzipping
# an package sdist and prepping for a sphinx execution.

import glob
import logging
import shutil
import argparse
from pkg_resources import Requirement
import ast
import os
import textwrap
import io
from tox_helper_tasks import (
    unzip_sdist_to_directory,
    move_and_rename
)

from ci_tools.parsing import ParsedSetup

logging.getLogger().setLevel(logging.INFO)

RST_EXTENSION_FOR_INDEX = """

## Indices and tables

- {{ref}}`genindex`
- {{ref}}`modindex`
- {{ref}}`search`

```{{toctree}}
:caption: Developer Documentation
:glob: true
:maxdepth: 5

{}

```

"""

root_dir = os.path.abspath(os.path.join(os.path.abspath(__file__), "..", "..", ".."))
sphinx_conf = os.path.join(root_dir, "doc", "sphinx", "individual_build_conf.py")


def should_build_docs(package_name):
    return not ("nspkg" in package_name or package_name in ["azure", "azure-mgmt", "azure-keyvault", "azure-documentdb", "azure-mgmt-documentdb", "azure-servicemanagement-legacy", "azure-core-tracing-opencensus"])

def create_index_file(readme_location, package_rst):
    readme_ext = os.path.splitext(readme_location)[1]

    if readme_ext == ".md":
        with open(readme_location, "r") as file:
            output = file.read()
    else:
        logging.error(
            "{} is not a valid readme type. Expecting RST or MD.".format(
                readme_location
            )
        )

    output += RST_EXTENSION_FOR_INDEX.format(package_rst)

    return output


def copy_conf(doc_folder):
    if not os.path.exists(doc_folder):
        os.mkdir(doc_folder)

    shutil.copy(sphinx_conf, os.path.join(doc_folder, 'conf.py'))


def create_index(doc_folder, source_location, namespace):
    index_content = ""

    package_rst = "{}.rst".format(namespace)
    content_destination = os.path.join(doc_folder, "index.md")

    if not os.path.exists(doc_folder):
        os.mkdir(doc_folder)

    # grep all content
    markdown_readmes = glob.glob(os.path.join(source_location, "README.md"))

    # if markdown, take that, otherwise rst
    if markdown_readmes:
        index_content = create_index_file(markdown_readmes[0], package_rst)
    else:
        logging.warning("No readmes detected for this namespace {}".format(namespace))
        index_content = RST_EXTENSION_FOR_INDEX.format(package_rst)

    # write index
    with open(content_destination, "w+", encoding='utf-8') as f:
        f.write(index_content)


def write_version(site_folder, version):

    if not os.path.isdir(site_folder):
        os.mkdir(site_folder)

    with open(os.path.join(site_folder, "version.txt"), "w") as f:
        f.write(version)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description="Prep a doc folder for consumption by sphinx."
    )

    parser.add_argument(
        "-d",
        "--dist_dir",
        dest="dist_dir",
        help="The dist location on disk. Usually /tox/dist.",
        required=True,
    )

    parser.add_argument(
        "-t",
        "--target",
        dest="target_package",
        help="The target package directory on disk. The target module passed to pylint will be <target_package>/azure.",
        required=True,
    )

    args = parser.parse_args()

    package_path = os.path.abspath(args.target_package)
    pkg_details = ParsedSetup.from_path(package_path)

    if should_build_docs(pkg_details.name):
        source_location = move_and_rename(unzip_sdist_to_directory(args.dist_dir))
        doc_folder = os.path.join(source_location, "docgen")

        create_index(doc_folder, source_location, pkg_details.namespace)

        site_folder = os.path.join(args.dist_dir, "site")
        write_version(site_folder, pkg_details.version)
    else:
        logging.info("Skipping sphinx prep for {}".format(pkg_details.name))


# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------
import pytest
from unittest.mock import patch, call
from azure.appconfiguration.provider.aio._async_discovery import (
    _get_known_domain,
    _request_record,
    _find_replicas,
    _find_origin,
    find_auto_failover_endpoints,
)
from dns.resolver import NXDOMAIN, YXDOMAIN, LifetimeTimeout, NoNameservers, Answer  # cspell:disable-line

AZCONFIG_IO = ".azconfig.io"  # cspell:disable-line
APPCONFIG_IO = ".appconfig.io"  # cspell:disable-line


class FakeAnswer:

    def __init__(self, priority, weight, port, target):
        self.priority = priority
        self.weight = weight
        self.port = port
        self.target = target.rstrip(".")


@pytest.mark.usefixtures("caplog")
class TestDiscovery:

    def test_get_known_domain(self):
        fake_endpoint = "https://fake.endpoint"
        assert _get_known_domain(fake_endpoint) is None
        fake_endpoint = "https://fake.endpoint." + AZCONFIG_IO  # cspell:disable-line
        assert _get_known_domain(fake_endpoint) == AZCONFIG_IO  # cspell:disable-line
        fake_endpoint = "https://fake.endpoint" + AZCONFIG_IO + ".test"  # cspell:disable-line
        assert _get_known_domain(fake_endpoint) == AZCONFIG_IO + ".test"  # cspell:disable-line
        fake_endpoint = "https://fake.endpoint" + APPCONFIG_IO
        assert _get_known_domain(fake_endpoint) == APPCONFIG_IO

    @pytest.mark.asyncio
    @patch("azure.appconfiguration.provider.aio._async_discovery.dns.asyncresolver.resolve")
    async def test_request_record(self, mock_dns):
        origin_request = "_origin._tcp.fake.endpoint"
        mock_dns.return_value = []
        assert len(await _request_record(origin_request)) == 0

        mock_dns.return_value = [1, 2, 3]
        assert len(await _request_record(origin_request)) == 3

        mock_dns.side_effect = NXDOMAIN  # cspell:disable-line
        assert len(await _request_record(origin_request)) == 0

        mock_dns.side_effect = YXDOMAIN  # cspell:disable-line
        assert len(await _request_record(origin_request)) == 0

        mock_dns.side_effect = LifetimeTimeout
        assert not await _request_record(origin_request)

        mock_dns.side_effect = NoNameservers
        assert not await _request_record(origin_request)

    @pytest.mark.asyncio
    @patch("azure.appconfiguration.provider.aio._async_discovery._request_record")
    async def test_find_replicas(self, mock_request_record):
        origin = "fake.endpoint"
        mock_request_record.return_value = None
        assert not await _find_replicas(origin)
        mock_request_record.assert_called_once_with("_alt0._tcp.fake.endpoint")

        mock_request_record.reset_mock()
        mock_request_record.return_value = []
        assert len(await _find_replicas(origin)) == 0
        mock_request_record.assert_called_once_with("_alt0._tcp.fake.endpoint")

        mock_request_record.reset_mock()
        mock_request_record.side_effect = [[FakeAnswer(1, 99, 5000, "fake.endpoint.")], []]
        result = await _find_replicas(origin)
        assert len(result) == 1
        assert result[0].priority == 1
        assert result[0].weight == 99
        assert result[0].port == 5000
        assert result[0].target == "fake.endpoint"
        assert mock_request_record.call_count == 2
        mock_request_record.assert_has_calls([call("_alt0._tcp.fake.endpoint"), call("_alt1._tcp.fake.endpoint")])

        mock_request_record.reset_mock()
        mock_request_record.side_effect = [
            [FakeAnswer(1, 99, 5000, "fake.endpoint."), FakeAnswer(2, 98, 5001, "fake.endpoint.")],
            [],
        ]
        result = await _find_replicas(origin)
        assert len(result) == 2
        assert mock_request_record.call_count == 2
        mock_request_record.assert_has_calls([call("_alt0._tcp.fake.endpoint"), call("_alt1._tcp.fake.endpoint")])

        mock_request_record.reset_mock()
        mock_request_record.side_effect = [
            [FakeAnswer(1, 99, 5000, "fake.endpoint."), FakeAnswer(2, 98, 5001, "fake.endpoint.")],
            [FakeAnswer(3, 99, 5000, "fake.endpoint.")],
            [],
        ]
        result = await _find_replicas(origin)
        assert len(result) == 3
        assert mock_request_record.call_count == 3
        mock_request_record.assert_has_calls(
            [call("_alt0._tcp.fake.endpoint"), call("_alt1._tcp.fake.endpoint"), call("_alt2._tcp.fake.endpoint")]
        )

    @pytest.mark.asyncio
    @patch("azure.appconfiguration.provider.aio._async_discovery._request_record")
    async def test_find_origin(self, mock_request_record):
        endpoint = "https://fake.endpoint"
        mock_request_record.return_value = None
        assert not await _find_origin(endpoint)
        mock_request_record.assert_called_once_with("_origin._tcp.fake.endpoint")

        mock_request_record.reset_mock()
        mock_request_record.return_value = []
        assert not await _find_origin(endpoint)
        mock_request_record.assert_called_once_with("_origin._tcp.fake.endpoint")

        mock_request_record.reset_mock()
        mock_request_record.return_value = [FakeAnswer(1, 99, 5000, "fake.endpoint.")]
        result = await _find_origin(endpoint)
        assert result.priority == 1
        assert result.weight == 99
        assert result.port == 5000
        assert result.target == "fake.endpoint"
        mock_request_record.assert_called_once_with("_origin._tcp.fake.endpoint")

        mock_request_record.reset_mock()
        mock_request_record.return_value = [
            FakeAnswer(1, 99, 5000, "fake.endpoint1."),
            FakeAnswer(2, 98, 5001, "fake.endpoint2."),
        ]
        result = await _find_origin(endpoint)
        assert result.priority == 1
        assert result.weight == 99
        assert result.port == 5000
        assert result.target == "fake.endpoint1"
        mock_request_record.assert_called_once_with("_origin._tcp.fake.endpoint")

    @pytest.mark.asyncio
    @patch("azure.appconfiguration.provider.aio._async_discovery._find_origin")
    @patch("azure.appconfiguration.provider.aio._async_discovery._find_replicas")
    async def test_find_auto_failover_endpoints(self, mock_find_replicas, mock_find_origin):
        endpoint = "https://fake.appconfig.io"

        assert len(await find_auto_failover_endpoints(endpoint, False)) == 0
        mock_find_origin.assert_not_called()
        mock_find_replicas.assert_not_called()

        mock_find_origin.reset_mock()
        mock_find_replicas.reset_mock()
        assert len(await find_auto_failover_endpoints("bad.endpoint", True)) == 0
        mock_find_origin.assert_not_called()
        mock_find_replicas.assert_not_called()

        mock_find_origin.reset_mock()
        mock_find_replicas.reset_mock()
        mock_find_origin.return_value = FakeAnswer(1, 99, 5000, "fake.appconfig.io")
        mock_find_replicas.return_value = []
        assert not await find_auto_failover_endpoints(endpoint, True)

        mock_find_origin.reset_mock()
        mock_find_replicas.reset_mock()
        mock_find_origin.return_value = FakeAnswer(1, 99, 5000, "fake.appconfig.io")
        mock_find_replicas.return_value = [FakeAnswer(2, 98, 5001, "fake1.appconfig.io.")]
        result = await find_auto_failover_endpoints(endpoint, True)
        assert len(result) == 1
        assert result[0] == "https://fake1.appconfig.io"
        mock_find_origin.assert_called_once_with(endpoint)
        mock_find_replicas.assert_called_once_with("fake.appconfig.io")

        mock_find_origin.reset_mock()
        mock_find_replicas.reset_mock()
        mock_find_origin.return_value = FakeAnswer(1, 99, 5000, "fake.appconfig.io")
        mock_find_replicas.return_value = [
            FakeAnswer(2, 98, 5001, "fake1.appconfig.io."),
            FakeAnswer(3, 97, 5002, "fake2.appconfig.io."),
        ]
        result = await find_auto_failover_endpoints(endpoint, True)
        assert len(result) == 2
        assert result[0] == "https://fake1.appconfig.io"
        assert result[1] == "https://fake2.appconfig.io"
        mock_find_origin.assert_called_once_with(endpoint)
        mock_find_replicas.assert_called_once_with("fake.appconfig.io")

        mock_find_origin.reset_mock()
        mock_find_replicas.reset_mock()
        mock_find_origin.return_value = FakeAnswer(1, 99, 5000, "fake.appconfig.io")
        mock_find_replicas.return_value = [
            FakeAnswer(2, 98, 5001, "fake1.appconfig.io."),
            FakeAnswer(3, 97, 5002, "fake2.appconfig.io."),
        ]
        result = await find_auto_failover_endpoints("https://fake1.appconfig.io", True)
        assert len(result) == 2
        assert result[0] == "https://fake.appconfig.io"
        assert result[1] == "https://fake2.appconfig.io"
        mock_find_origin.assert_called_once_with("https://fake1.appconfig.io")
        mock_find_replicas.assert_called_once_with("fake.appconfig.io")



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------
from typing import Any, Callable, AsyncIterator, Iterator, Optional, cast
from types import TracebackType

from opentelemetry.trace import Span, StatusCode

from azure.core.tracing._abstract_span import AbstractSpan
from azure.core.async_paging import AsyncItemPaged
from azure.core.paging import ItemPaged


class _SpanLogger:
    """The class, providing safe logging of events to the span"""

    def log_to_span_safe(
        self,
        val: Any,
        instrumentation_fun: Callable[[AbstractSpan, Any], None],
        span: AbstractSpan,
    ) -> None:
        """
        Log value to the span if span exists.

        :param val: The value to be logged.
        :type val: Any
        :param instrumentation_fun: The function to be used to log val.
        :type instrumentation_fun: Callable[[AbstractSpan, Any], None]
        :param span: The span to be used for logging. Span must be opened before calling this method.
        :type span: AbstractSpan
        """
        try:
            instrumentation_fun(span, val)

        except Exception as exc:
            # Set the span status to error
            if isinstance(span.span_instance, Span):  # pyright: ignore [reportPossiblyUnboundVariable]
                span.span_instance.set_status(
                    StatusCode.ERROR,  # pyright: ignore [reportPossiblyUnboundVariable]
                    description=str(exc),
                )
            module = getattr(exc, "__module__", "")
            module = module if module != "builtins" else ""
            error_type = f"{module}.{type(exc).__name__}" if module else type(exc).__name__
            span.add_attribute("error.type", error_type)
            raise


class _AsyncInstrumentedItemPaged(AsyncItemPaged, _SpanLogger):
    """The list class to mimic the AsyncPageable returned by a list."""

    def __init__(
        self,
        async_iter: AsyncItemPaged,
        instrumentation_fun: Callable[[AbstractSpan, Any], None],
        span: Optional[AbstractSpan],
    ) -> None:
        super().__init__()
        self._iter = async_iter
        self._inst_fun = instrumentation_fun
        self._span = span
        self._gen: Optional[AsyncIterator[Any]] = None

    def __getattr__(self, name: str) -> Any:
        """
        Delegate every attribute we do not override to the wrapped object

        :param name: The name of the attribute to get.
        :type name: str

        :return: The value of the attribute.
        :rtype: Any
        """
        return getattr(self._iter, name)

    def __aiter__(self) -> AsyncIterator[Any]:
        async def _gen() -> AsyncIterator[Any]:
            try:
                async for val in self._iter:
                    if self._span is not None:
                        self.log_to_span_safe(val, self._inst_fun, self._span)
                    yield val
            finally:
                if self._span is not None:
                    # We cast None to TracebackType, because traceback is
                    # not used in the downstream code.
                    self._span.__exit__(None, None, cast(TracebackType, None))

        if self._gen is None:
            if self._span is not None:
                self._span.__enter__()
            self._gen = _gen()
        return self._gen


class _InstrumentedItemPaged(ItemPaged, _SpanLogger):
    """The list class to mimic the Pageable returned by a list."""

    def __init__(
        self,
        iter_val: ItemPaged,
        instrumentation_fun: Callable[[AbstractSpan, Any], None],
        span: Optional[AbstractSpan],
    ) -> None:
        super().__init__()
        self._iter = iter_val
        self._inst_fun = instrumentation_fun
        self._span = span
        self._gen: Optional[Iterator[Any]] = None

    def __getattr__(self, name: str) -> Any:
        """
        Delegate every attribute we do not override to the wrapped object

        :param name: The name of the attribute to get.
        :type name: str

        :return: The value of the attribute.
        :rtype: Any
        """
        return getattr(self._iter, name)

    def __iter__(self) -> Iterator[Any]:
        def _gen() -> Iterator[Any]:
            try:
                for val in self._iter:
                    if self._span is not None:
                        self.log_to_span_safe(val, self._inst_fun, self._span)
                    yield val
            finally:
                if self._span is not None:
                    # We cast None to TracebackType, because traceback is
                    # not used in the downstream code.
                    self._span.__exit__(None, None, cast(TracebackType, None))

        if self._gen is None:
            if self._span is not None:
                self._span.__enter__()
            self._gen = _gen()
        return self._gen



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.deviceregistry import DeviceRegistryMgmtClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer, recorded_by_proxy

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestDeviceRegistryMgmtAssetEndpointProfilesOperations(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(DeviceRegistryMgmtClient)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_asset_endpoint_profiles_get(self, resource_group):
        response = self.client.asset_endpoint_profiles.get(
            resource_group_name=resource_group.name,
            asset_endpoint_profile_name="str",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_asset_endpoint_profiles_begin_create_or_replace(self, resource_group):
        response = self.client.asset_endpoint_profiles.begin_create_or_replace(
            resource_group_name=resource_group.name,
            asset_endpoint_profile_name="str",
            resource={
                "extendedLocation": {"name": "str", "type": "str"},
                "location": "str",
                "id": "str",
                "name": "str",
                "properties": {
                    "endpointProfileType": "str",
                    "targetAddress": "str",
                    "additionalConfiguration": "str",
                    "authentication": {
                        "method": "str",
                        "usernamePasswordCredentials": {"passwordSecretName": "str", "usernameSecretName": "str"},
                        "x509Credentials": {"certificateSecretName": "str"},
                    },
                    "discoveredAssetEndpointProfileRef": "str",
                    "provisioningState": "str",
                    "status": {"errors": [{"code": 0, "message": "str"}]},
                    "uuid": "str",
                },
                "systemData": {
                    "createdAt": "2020-02-20 00:00:00",
                    "createdBy": "str",
                    "createdByType": "str",
                    "lastModifiedAt": "2020-02-20 00:00:00",
                    "lastModifiedBy": "str",
                    "lastModifiedByType": "str",
                },
                "tags": {"str": "str"},
                "type": "str",
            },
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_asset_endpoint_profiles_begin_update(self, resource_group):
        response = self.client.asset_endpoint_profiles.begin_update(
            resource_group_name=resource_group.name,
            asset_endpoint_profile_name="str",
            properties={
                "properties": {
                    "additionalConfiguration": "str",
                    "authentication": {
                        "method": "str",
                        "usernamePasswordCredentials": {"passwordSecretName": "str", "usernameSecretName": "str"},
                        "x509Credentials": {"certificateSecretName": "str"},
                    },
                    "endpointProfileType": "str",
                    "targetAddress": "str",
                },
                "tags": {"str": "str"},
            },
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_asset_endpoint_profiles_begin_delete(self, resource_group):
        response = self.client.asset_endpoint_profiles.begin_delete(
            resource_group_name=resource_group.name,
            asset_endpoint_profile_name="str",
        ).result()  # call '.result()' to poll until service return final result

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_asset_endpoint_profiles_list_by_resource_group(self, resource_group):
        response = self.client.asset_endpoint_profiles.list_by_resource_group(
            resource_group_name=resource_group.name,
        )
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy
    def test_asset_endpoint_profiles_list_by_subscription(self, resource_group):
        response = self.client.asset_endpoint_profiles.list_by_subscription()
        result = [r for r in response]
        # please add some check logic here by yourself
        # ...



# ------------------------------------
# Copyright (c) Microsoft Corporation.
# Licensed under the MIT License.
# ------------------------------------

"""
DESCRIPTION:
    This sample demonstrates how to use agent operations with toolset and iteration in streaming from
    the Azure Agents service using a synchronous client.

USAGE:
    python sample_agents_stream_iteration_with_toolset.py

    Before running the sample:

    pip install azure-ai-agents azure-identity

    Set these environment variables with your own values:
    1) PROJECT_ENDPOINT - The Azure AI Project endpoint, as found in the Overview
                          page of your Azure AI Foundry portal.
    2) MODEL_DEPLOYMENT_NAME - The deployment name of the AI model, as found under the "Name" column in
       the "Models + endpoints" tab in your Azure AI Foundry project.
"""

import os, sys
from azure.ai.agents import AgentsClient
from azure.ai.agents.models import AgentStreamEvent, RunStepDeltaChunk
from azure.ai.agents.models import (
    MessageDeltaChunk,
    ListSortOrder,
    RunStep,
    ThreadMessage,
    ThreadRun,
)
from azure.ai.agents.models import FunctionTool, ToolSet
from azure.identity import DefaultAzureCredential

current_path = os.path.dirname(__file__)
root_path = os.path.abspath(os.path.join(current_path, os.pardir, os.pardir))
if root_path not in sys.path:
    sys.path.insert(0, root_path)
from samples.utils.user_functions import user_functions

agents_client = AgentsClient(
    endpoint=os.environ["PROJECT_ENDPOINT"],
    credential=DefaultAzureCredential(),
)

functions = FunctionTool(user_functions)
toolset = ToolSet()
toolset.add(functions)

with agents_client:
    agents_client.enable_auto_function_calls(toolset)
    agent = agents_client.create_agent(
        model=os.environ["MODEL_DEPLOYMENT_NAME"],
        name="my-agent",
        instructions="You are a helpful agent",
        toolset=toolset,
    )
    print(f"Created agent, agent ID: {agent.id}")

    thread = agents_client.threads.create()
    print(f"Created thread, thread ID {thread.id}")

    message = agents_client.messages.create(thread_id=thread.id, role="user", content="Hello, what's the time?")
    print(f"Created message, message ID {message.id}")

    with agents_client.runs.stream(thread_id=thread.id, agent_id=agent.id) as stream:

        for event_type, event_data, _ in stream:

            if isinstance(event_data, MessageDeltaChunk):
                print(f"Text delta received: {event_data.text}")

            elif isinstance(event_data, RunStepDeltaChunk):
                print(f"RunStepDeltaChunk received. ID: {event_data.id}.")

            elif isinstance(event_data, ThreadMessage):
                print(f"ThreadMessage created. ID: {event_data.id}, Status: {event_data.status}")

            elif isinstance(event_data, ThreadRun):
                print(f"ThreadRun status: {event_data.status}")

                if event_data.status == "failed":
                    print(f"Run failed. Error: {event_data.last_error}")

            elif isinstance(event_data, RunStep):
                print(f"RunStep type: {event_data.type}, Status: {event_data.status}")

            elif event_type == AgentStreamEvent.ERROR:
                print(f"An error occurred. Data: {event_data}")

            elif event_type == AgentStreamEvent.DONE:
                print("Stream completed.")

            else:
                print(f"Unhandled Event Type: {event_type}, Data: {event_data}")

    agents_client.delete_agent(agent.id)
    print("Deleted agent")

    messages = agents_client.messages.list(thread_id=thread.id, order=ListSortOrder.ASCENDING)
    for msg in messages:
        if msg.text_messages:
            last_text = msg.text_messages[-1]
            print(f"{msg.role}: {last_text.text.value}")



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from azure.identity import DefaultAzureCredential

from azure.mgmt.appcontainers import ContainerAppsAPIClient

"""
# PREREQUISITES
    pip install azure-identity
    pip install azure-mgmt-appcontainers
# USAGE
    python session_pools_lifecycle_timed_create_or_update.py

    Before run the sample, please set the values of the client ID, tenant ID and client secret
    of the AAD application as environment variables: AZURE_CLIENT_ID, AZURE_TENANT_ID,
    AZURE_CLIENT_SECRET. For more info about how to get the value, please see:
    https://docs.microsoft.com/azure/active-directory/develop/howto-create-service-principal-portal
"""


def main():
    client = ContainerAppsAPIClient(
        credential=DefaultAzureCredential(),
        subscription_id="34adfa4f-cedf-4dc0-ba29-b6d1a69ab345",
    )

    response = client.container_apps_session_pools.begin_create_or_update(
        resource_group_name="rg",
        session_pool_name="testsessionpool",
        session_pool_envelope={
            "identity": {"type": "SystemAssigned"},
            "location": "East US",
            "properties": {
                "containerType": "CustomContainer",
                "customContainerTemplate": {
                    "containers": [
                        {
                            "args": ["-c", "while true; do echo hello; sleep 10;done"],
                            "command": ["/bin/sh"],
                            "image": "repo/testcontainer:v4",
                            "name": "testinitcontainer",
                            "resources": {"cpu": 0.25, "memory": "0.5Gi"},
                        }
                    ],
                    "ingress": {"targetPort": 80},
                    "registryCredentials": {
                        "identity": "/subscriptions/7a497526-bb8d-4816-9795-db1418a1f977/resourcegroups/test/providers/Microsoft.ManagedIdentity/userAssignedIdentities/testSP",
                        "server": "test.azurecr.io",
                    },
                },
                "dynamicPoolConfiguration": {
                    "lifecycleConfiguration": {"lifecycleType": "OnContainerExit", "maxAlivePeriodInSeconds": 86400}
                },
                "environmentId": "/subscriptions/34adfa4f-cedf-4dc0-ba29-b6d1a69ab345/resourceGroups/rg/providers/Microsoft.App/managedEnvironments/demokube",
                "managedIdentitySettings": [{"identity": "system", "lifecycle": "Main"}],
                "poolManagementType": "Dynamic",
                "scaleConfiguration": {"maxConcurrentSessions": 500, "readySessionInstances": 100},
                "sessionNetworkConfiguration": {"status": "EgressEnabled"},
            },
        },
    ).result()
    print(response)


# x-ms-original-file: specification/app/resource-manager/Microsoft.App/stable/2025-01-01/examples/SessionPools_LifecycleTimed_CreateOrUpdate.json
if __name__ == "__main__":
    main()



# -------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for
# license information.
# --------------------------------------------------------------------------

import pytest
import azure.communication.phonenumbers._shared.models as models

test_user_id = "test_user_id"
test_unknown_id = "test_unknown_id"
test_phone_number = "+123456789101"


def test_communication_user_identifier():
    comm_user = models.CommunicationUserIdentifier(test_user_id)
    assert comm_user.kind == models.CommunicationIdentifierKind.COMMUNICATION_USER
    assert comm_user.raw_id == test_user_id
    assert comm_user.properties["id"] == test_user_id


def test_phone_number_identifier():
    phone_num = models.PhoneNumberIdentifier(test_phone_number)
    assert phone_num.kind == models.CommunicationIdentifierKind.PHONE_NUMBER
    assert phone_num.raw_id == f"{models.PHONE_NUMBER_PREFIX}{test_phone_number}"
    assert phone_num.properties["value"] == test_phone_number


def test_unknown_identifier():
    unknown = models.UnknownIdentifier(test_unknown_id)
    assert unknown.kind == models.CommunicationIdentifierKind.UNKNOWN
    assert unknown.raw_id == test_unknown_id
    assert unknown.properties == {}


def test_microsoft_teams_user_identifier():
    teams_user = models.MicrosoftTeamsUserIdentifier(test_user_id)
    assert teams_user.kind == models.CommunicationIdentifierKind.MICROSOFT_TEAMS_USER
    assert teams_user.raw_id == f"{models.TEAMS_USER_PUBLIC_CLOUD_PREFIX}{test_user_id}"
    assert teams_user.properties["user_id"] == test_user_id
    assert teams_user.properties["is_anonymous"] == False
    assert teams_user.properties["cloud"] == models.CommunicationCloudEnvironment.PUBLIC


def test_identifier_from_raw_id_phone_number():
    raw_id = f"{models.PHONE_NUMBER_PREFIX}{test_phone_number}"
    identifier = models.identifier_from_raw_id(raw_id)
    assert isinstance(identifier, models.PhoneNumberIdentifier)


def test_identifier_from_raw_id_teams_user():
    raw_id = f"{models.TEAMS_USER_PUBLIC_CLOUD_PREFIX}{test_user_id}"
    identifier = models.identifier_from_raw_id(raw_id)
    assert isinstance(identifier, models.MicrosoftTeamsUserIdentifier)


def test_identifier_from_raw_id_unknown():
    identifier = models.identifier_from_raw_id(test_unknown_id)
    assert isinstance(identifier, models.UnknownIdentifier)


def test_microsoft_teams_user_identifier_anonymous():
    teams_user = models.MicrosoftTeamsUserIdentifier(test_user_id, is_anonymous=True)
    assert teams_user.kind == models.CommunicationIdentifierKind.MICROSOFT_TEAMS_USER
    assert teams_user.raw_id == f"{models.TEAMS_USER_ANONYMOUS_PREFIX}{test_user_id}"
    assert teams_user.properties["user_id"] == test_user_id
    assert teams_user.properties["is_anonymous"] == True
    assert teams_user.properties["cloud"] == models.CommunicationCloudEnvironment.PUBLIC


def test_microsoft_teams_user_identifier_cloud_types():
    for cloud, prefix in zip(
        [models.CommunicationCloudEnvironment.DOD, models.CommunicationCloudEnvironment.GCCH],
        [models.TEAMS_USER_DOD_CLOUD_PREFIX, models.TEAMS_USER_GCCH_CLOUD_PREFIX],
    ):
        teams_user = models.MicrosoftTeamsUserIdentifier(test_user_id, cloud=cloud)
        assert teams_user.kind == models.CommunicationIdentifierKind.MICROSOFT_TEAMS_USER
        assert teams_user.raw_id == f"{prefix}{test_user_id}"
        assert teams_user.properties["user_id"] == test_user_id
        assert teams_user.properties["is_anonymous"] == False
        assert teams_user.properties["cloud"] == cloud


def test_identifier_from_raw_id_microsoft_teams_user():
    for prefix in [
        models.TEAMS_USER_ANONYMOUS_PREFIX,
        models.TEAMS_USER_PUBLIC_CLOUD_PREFIX,
        models.TEAMS_USER_DOD_CLOUD_PREFIX,
        models.TEAMS_USER_GCCH_CLOUD_PREFIX,
    ]:
        raw_id = f"{prefix}:{test_user_id}"
        identifier = models.identifier_from_raw_id(raw_id)
        assert isinstance(identifier, models.MicrosoftTeamsUserIdentifier)


def test_identifier_from_raw_id_communication_user():
    for prefix in [
        models.ACS_USER_PREFIX,
        models.ACS_USER_DOD_CLOUD_PREFIX,
        models.ACS_USER_GCCH_CLOUD_PREFIX,
        models.SPOOL_USER_PREFIX,
    ]:
        raw_id = f"{prefix}:{test_user_id}"
        identifier = models.identifier_from_raw_id(raw_id)
        assert isinstance(identifier, models.CommunicationUserIdentifier)


def test_communication_user_identifier_equality():
    user1 = models.CommunicationUserIdentifier(test_user_id)
    user2 = models.CommunicationUserIdentifier(test_user_id)
    user3 = models.CommunicationUserIdentifier(test_unknown_id)
    assert user1 == user2
    assert user1 != user3


def test_phone_number_identifier_equality():
    phone1 = models.PhoneNumberIdentifier(test_phone_number)
    phone2 = models.PhoneNumberIdentifier(test_phone_number)
    phone3 = models.PhoneNumberIdentifier("+0987654321")
    assert phone1 == phone2
    assert phone1 != phone3


def test_microsoft_teams_user_identifier_equality():
    user1 = models.MicrosoftTeamsUserIdentifier(test_user_id)
    user2 = models.MicrosoftTeamsUserIdentifier(test_user_id)
    user3 = models.MicrosoftTeamsUserIdentifier(test_unknown_id)
    assert user1 == user2
    assert user1 != user3



# --------------------------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# --------------------------------------------------------------------------------------------

from time import time
from wsgiref.handlers import format_date_time
from devtools_testutils.perfstress_tests import RandomStream, AsyncRandomStream

from azure.core.rest import HttpRequest
from azure.core.exceptions import (
    HttpResponseError,
    map_error,
)
from ._test_base import _BlobTest


class UploadBinaryDataTest(_BlobTest):
    def __init__(self, arguments):
        super().__init__(arguments)
        blob_name = "uploadtest"
        self.blob_endpoint = f"{self.account_endpoint}{self.container_name}/{blob_name}"
        self.upload_stream = RandomStream(self.args.size)
        self.upload_stream_async = AsyncRandomStream(self.args.size)

    def run_sync(self):
        self.upload_stream.reset()
        current_time = format_date_time(time())
        request = HttpRequest(
            method="PUT",
            url=self.blob_endpoint,
            params={},
            headers={
                "x-ms-date": current_time,
                "x-ms-blob-type": "BlockBlob",
                "Content-Length": str(self.args.size),
                "x-ms-version": self.api_version,
                "Content-Type": "application/octet-stream",
            },
            content=self.upload_stream,
        )
        response = self.pipeline_client._pipeline.run(request).http_response
        if response.status_code not in [201]:
            map_error(status_code=response.status_code, response=response, error_map=self.error_map)
            raise HttpResponseError(response=response)

    async def run_async(self):
        self.upload_stream_async.reset()
        current_time = format_date_time(time())
        request = HttpRequest(
            method="PUT",
            url=self.blob_endpoint,
            params={},
            headers={
                "x-ms-date": current_time,
                "x-ms-blob-type": "BlockBlob",
                "Content-Length": str(self.args.size),
                "x-ms-version": self.api_version,
                "Content-Type": "application/octet-stream",
            },
            content=self.upload_stream_async,
        )
        pipeline_response = await self.async_pipeline_client._pipeline.run(request)
        response = pipeline_response.http_response
        if response.status_code not in [201]:
            map_error(status_code=response.status_code, response=response, error_map=self.error_map)
            raise HttpResponseError(response=response)

    async def close(self):
        await super().close()



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import pytest
from azure.mgmt.apimanagement.aio import ApiManagementClient

from devtools_testutils import AzureMgmtRecordedTestCase, RandomNameResourceGroupPreparer
from devtools_testutils.aio import recorded_by_proxy_async

AZURE_LOCATION = "eastus"


@pytest.mark.skip("you may need to update the auto-generated test case before run it")
class TestApiManagementWorkspaceOperationsAsync(AzureMgmtRecordedTestCase):
    def setup_method(self, method):
        self.client = self.create_mgmt_client(ApiManagementClient, is_async=True)

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workspace_list_by_service(self, resource_group):
        response = self.client.workspace.list_by_service(
            resource_group_name=resource_group.name,
            service_name="str",
            api_version="2024-05-01",
        )
        result = [r async for r in response]
        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workspace_get_entity_tag(self, resource_group):
        response = await self.client.workspace.get_entity_tag(
            resource_group_name=resource_group.name,
            service_name="str",
            workspace_id="str",
            api_version="2024-05-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workspace_get(self, resource_group):
        response = await self.client.workspace.get(
            resource_group_name=resource_group.name,
            service_name="str",
            workspace_id="str",
            api_version="2024-05-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workspace_create_or_update(self, resource_group):
        response = await self.client.workspace.create_or_update(
            resource_group_name=resource_group.name,
            service_name="str",
            workspace_id="str",
            parameters={"description": "str", "displayName": "str", "id": "str", "name": "str", "type": "str"},
            api_version="2024-05-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workspace_update(self, resource_group):
        response = await self.client.workspace.update(
            resource_group_name=resource_group.name,
            service_name="str",
            workspace_id="str",
            if_match="str",
            parameters={"description": "str", "displayName": "str", "id": "str", "name": "str", "type": "str"},
            api_version="2024-05-01",
        )

        # please add some check logic here by yourself
        # ...

    @RandomNameResourceGroupPreparer(location=AZURE_LOCATION)
    @recorded_by_proxy_async
    async def test_workspace_delete(self, resource_group):
        response = await self.client.workspace.delete(
            resource_group_name=resource_group.name,
            service_name="str",
            workspace_id="str",
            if_match="str",
            api_version="2024-05-01",
        )

        # please add some check logic here by yourself
        # ...



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) Python Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------
import datetime
from io import IOBase
import json
import sys
from typing import Any, Callable, Dict, IO, Iterable, Iterator, List, Optional, TypeVar, Union, overload
import urllib.parse
import uuid

from azure.core.exceptions import (
    ClientAuthenticationError,
    HttpResponseError,
    ResourceExistsError,
    ResourceNotFoundError,
    ResourceNotModifiedError,
    StreamClosedError,
    StreamConsumedError,
    map_error,
)
from azure.core.paging import ItemPaged
from azure.core.pipeline import PipelineResponse
from azure.core.rest import HttpRequest, HttpResponse
from azure.core.tracing.decorator import distributed_trace
from azure.core.utils import case_insensitive_dict

from .. import models as _models
from .._model_base import SdkJSONEncoder, _deserialize
from .._serialization import Serializer
from .._vendor import MessageTemplateClientMixinABC, NotificationMessagesClientMixinABC

if sys.version_info >= (3, 9):
    from collections.abc import MutableMapping
else:
    from typing import MutableMapping  # type: ignore
JSON = MutableMapping[str, Any]  # pylint: disable=unsubscriptable-object
T = TypeVar("T")
ClsType = Optional[Callable[[PipelineResponse[HttpRequest, HttpResponse], T, Dict[str, Any]], Any]]

_SERIALIZER = Serializer()
_SERIALIZER.client_side_validation = False


def build_notification_messages_send_request(**kwargs: Any) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-01-15-preview"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/messages/notifications:send"

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    if "Repeatability-Request-ID" not in _headers:
        _headers["Repeatability-Request-ID"] = str(uuid.uuid4())
    if "Repeatability-First-Sent" not in _headers:
        _headers["Repeatability-First-Sent"] = _SERIALIZER.serialize_data(
            datetime.datetime.now(datetime.timezone.utc), "rfc-1123"
        )
    if content_type is not None:
        _headers["Content-Type"] = _SERIALIZER.header("content_type", content_type, "str")
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="POST", url=_url, params=_params, headers=_headers, **kwargs)


def build_notification_messages_download_media_request(  # pylint: disable=name-too-long
    id: str, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-01-15-preview"))
    accept = _headers.pop("Accept", "application/octet-stream")

    # Construct URL
    _url = "/messages/streams/{id}"
    path_format_arguments = {
        "id": _SERIALIZER.url("id", id, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


def build_message_template_list_templates_request(  # pylint: disable=name-too-long
    channel_id: str, *, maxpagesize: Optional[int] = None, **kwargs: Any
) -> HttpRequest:
    _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
    _params = case_insensitive_dict(kwargs.pop("params", {}) or {})

    api_version: str = kwargs.pop("api_version", _params.pop("api-version", "2025-01-15-preview"))
    accept = _headers.pop("Accept", "application/json")

    # Construct URL
    _url = "/messages/channels/{channelId}/templates"
    path_format_arguments = {
        "channelId": _SERIALIZER.url("channel_id", channel_id, "str"),
    }

    _url: str = _url.format(**path_format_arguments)  # type: ignore

    # Construct parameters
    _params["api-version"] = _SERIALIZER.query("api_version", api_version, "str")
    if maxpagesize is not None:
        _params["maxpagesize"] = _SERIALIZER.query("maxpagesize", maxpagesize, "int")

    # Construct headers
    _headers["Accept"] = _SERIALIZER.header("accept", accept, "str")

    return HttpRequest(method="GET", url=_url, params=_params, headers=_headers, **kwargs)


class NotificationMessagesClientOperationsMixin(NotificationMessagesClientMixinABC):  # pylint: disable=name-too-long

    @overload
    def send(
        self, body: _models.NotificationContent, *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.SendMessageResult:
        """Sends a notification message from Business to User.

        :param body: Details of the message to send. Required.
        :type body: ~azure.communication.messages.models.NotificationContent
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SendMessageResult. The SendMessageResult is compatible with MutableMapping
        :rtype: ~azure.communication.messages.models.SendMessageResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def send(self, body: JSON, *, content_type: str = "application/json", **kwargs: Any) -> _models.SendMessageResult:
        """Sends a notification message from Business to User.

        :param body: Details of the message to send. Required.
        :type body: JSON
        :keyword content_type: Body Parameter content-type. Content type parameter for JSON body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SendMessageResult. The SendMessageResult is compatible with MutableMapping
        :rtype: ~azure.communication.messages.models.SendMessageResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @overload
    def send(
        self, body: IO[bytes], *, content_type: str = "application/json", **kwargs: Any
    ) -> _models.SendMessageResult:
        """Sends a notification message from Business to User.

        :param body: Details of the message to send. Required.
        :type body: IO[bytes]
        :keyword content_type: Body Parameter content-type. Content type parameter for binary body.
         Default value is "application/json".
        :paramtype content_type: str
        :return: SendMessageResult. The SendMessageResult is compatible with MutableMapping
        :rtype: ~azure.communication.messages.models.SendMessageResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """

    @distributed_trace
    def send(
        self, body: Union[_models.NotificationContent, JSON, IO[bytes]], **kwargs: Any
    ) -> _models.SendMessageResult:
        """Sends a notification message from Business to User.

        :param body: Details of the message to send. Is one of the following types:
         NotificationContent, JSON, IO[bytes] Required.
        :type body: ~azure.communication.messages.models.NotificationContent or JSON or IO[bytes]
        :return: SendMessageResult. The SendMessageResult is compatible with MutableMapping
        :rtype: ~azure.communication.messages.models.SendMessageResult
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = case_insensitive_dict(kwargs.pop("headers", {}) or {})
        _params = kwargs.pop("params", {}) or {}

        content_type: Optional[str] = kwargs.pop("content_type", _headers.pop("Content-Type", None))
        cls: ClsType[_models.SendMessageResult] = kwargs.pop("cls", None)

        content_type = content_type or "application/json"
        _content = None
        if isinstance(body, (IOBase, bytes)):
            _content = body
        else:
            _content = json.dumps(body, cls=SdkJSONEncoder, exclude_readonly=True)  # type: ignore

        _request = build_notification_messages_send_request(
            content_type=content_type,
            api_version=self._config.api_version,
            content=_content,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", False)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [202]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        response_headers = {}
        response_headers["Repeatability-Result"] = self._deserialize(
            "str", response.headers.get("Repeatability-Result")
        )
        response_headers["x-ms-client-request-id"] = self._deserialize(
            "str", response.headers.get("x-ms-client-request-id")
        )

        if _stream:
            deserialized = response.iter_bytes()
        else:
            deserialized = _deserialize(_models.SendMessageResult, response.json())

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore

    @distributed_trace
    def download_media(self, id: str, **kwargs: Any) -> Iterator[bytes]:
        """Download the Media payload from a User to Business message.

        :param id: The stream ID. Required.
        :type id: str
        :return: Iterator[bytes]
        :rtype: Iterator[bytes]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        cls: ClsType[Iterator[bytes]] = kwargs.pop("cls", None)

        _request = build_notification_messages_download_media_request(
            id=id,
            api_version=self._config.api_version,
            headers=_headers,
            params=_params,
        )
        path_format_arguments = {
            "endpoint": self._serialize.url("self._config.endpoint", self._config.endpoint, "str", skip_quote=True),
        }
        _request.url = self._client.format_url(_request.url, **path_format_arguments)

        _stream = kwargs.pop("stream", True)
        pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
            _request, stream=_stream, **kwargs
        )

        response = pipeline_response.http_response

        if response.status_code not in [200]:
            if _stream:
                try:
                    response.read()  # Load the body in memory and close the socket
                except (StreamConsumedError, StreamClosedError):
                    pass
            map_error(status_code=response.status_code, response=response, error_map=error_map)
            raise HttpResponseError(response=response)

        response_headers = {}
        response_headers["x-ms-client-request-id"] = self._deserialize(
            "str", response.headers.get("x-ms-client-request-id")
        )
        response_headers["content-type"] = self._deserialize("str", response.headers.get("content-type"))

        deserialized = response.iter_bytes()

        if cls:
            return cls(pipeline_response, deserialized, response_headers)  # type: ignore

        return deserialized  # type: ignore


class MessageTemplateClientOperationsMixin(MessageTemplateClientMixinABC):

    @distributed_trace
    def list_templates(self, channel_id: str, **kwargs: Any) -> Iterable["_models.MessageTemplateItem"]:
        """List all templates for given Azure Communication Services channel.

        :param channel_id: The registration ID of the channel. Required.
        :type channel_id: str
        :return: An iterator like instance of MessageTemplateItem
        :rtype: ~azure.core.paging.ItemPaged[~azure.communication.messages.models.MessageTemplateItem]
        :raises ~azure.core.exceptions.HttpResponseError:
        """
        _headers = kwargs.pop("headers", {}) or {}
        _params = kwargs.pop("params", {}) or {}

        maxpagesize = kwargs.pop("maxpagesize", None)
        cls: ClsType[List[_models.MessageTemplateItem]] = kwargs.pop("cls", None)

        error_map: MutableMapping = {
            401: ClientAuthenticationError,
            404: ResourceNotFoundError,
            409: ResourceExistsError,
            304: ResourceNotModifiedError,
        }
        error_map.update(kwargs.pop("error_map", {}) or {})

        def prepare_request(next_link=None):
            if not next_link:

                _request = build_message_template_list_templates_request(
                    channel_id=channel_id,
                    maxpagesize=maxpagesize,
                    api_version=self._config.api_version,
                    headers=_headers,
                    params=_params,
                )
                path_format_arguments = {
                    "endpoint": self._serialize.url(
                        "self._config.endpoint", self._config.endpoint, "str", skip_quote=True
                    ),
                }
                _request.url = self._client.format_url(_request.url, **path_format_arguments)

            else:
                # make call to next link with the client's api-version
                _parsed_next_link = urllib.parse.urlparse(next_link)
                _next_request_params = case_insensitive_dict(
                    {
                        key: [urllib.parse.quote(v) for v in value]
                        for key, value in urllib.parse.parse_qs(_parsed_next_link.query).items()
                    }
                )
                _next_request_params["api-version"] = self._config.api_version
                _request = HttpRequest(
                    "GET", urllib.parse.urljoin(next_link, _parsed_next_link.path), params=_next_request_params
                )
                path_format_arguments = {
                    "endpoint": self._serialize.url(
                        "self._config.endpoint", self._config.endpoint, "str", skip_quote=True
                    ),
                }
                _request.url = self._client.format_url(_request.url, **path_format_arguments)

            return _request

        def extract_data(pipeline_response):
            deserialized = pipeline_response.http_response.json()
            list_of_elem = _deserialize(List[_models.MessageTemplateItem], deserialized["value"])
            if cls:
                list_of_elem = cls(list_of_elem)  # type: ignore
            return deserialized.get("nextLink") or None, iter(list_of_elem)

        def get_next(next_link=None):
            _request = prepare_request(next_link)

            _stream = False
            pipeline_response: PipelineResponse = self._client._pipeline.run(  # pylint: disable=protected-access
                _request, stream=_stream, **kwargs
            )
            response = pipeline_response.http_response

            if response.status_code not in [200]:
                map_error(status_code=response.status_code, response=response, error_map=error_map)
                raise HttpResponseError(response=response)

            return pipeline_response

        return ItemPaged(get_next, extract_data)



# coding=utf-8
# --------------------------------------------------------------------------
# Copyright (c) Microsoft Corporation. All rights reserved.
# Licensed under the MIT License. See License.txt in the project root for license information.
# Code generated by Microsoft (R) AutoRest Code Generator.
# Changes may cause incorrect behavior and will be lost if the code is regenerated.
# --------------------------------------------------------------------------

from ._operations import Operations
from ._access_review_schedule_definitions_operations import AccessReviewScheduleDefinitionsOperations
from ._access_review_instances_operations import AccessReviewInstancesOperations
from ._access_review_instance_operations import AccessReviewInstanceOperations
from ._access_review_instance_decisions_operations import AccessReviewInstanceDecisionsOperations
from ._access_review_default_settings_operations import AccessReviewDefaultSettingsOperations
from ._access_review_schedule_definitions_assigned_for_my_approval_operations import (
    AccessReviewScheduleDefinitionsAssignedForMyApprovalOperations,
)
from ._access_review_instances_assigned_for_my_approval_operations import (
    AccessReviewInstancesAssignedForMyApprovalOperations,
)
from ._access_review_instance_my_decisions_operations import AccessReviewInstanceMyDecisionsOperations

from ._patch import __all__ as _patch_all
from ._patch import *  # pylint: disable=unused-wildcard-import
from ._patch import patch_sdk as _patch_sdk

__all__ = [
    "Operations",
    "AccessReviewScheduleDefinitionsOperations",
    "AccessReviewInstancesOperations",
    "AccessReviewInstanceOperations",
    "AccessReviewInstanceDecisionsOperations",
    "AccessReviewDefaultSettingsOperations",
    "AccessReviewScheduleDefinitionsAssignedForMyApprovalOperations",
    "AccessReviewInstancesAssignedForMyApprovalOperations",
    "AccessReviewInstanceMyDecisionsOperations",
]
__all__.extend([p for p in _patch_all if p not in __all__])
_patch_sdk()



# --------------------------------------------------------------------------
#
# Copyright (c) Microsoft Corporation. All rights reserved.
#
# The MIT License (MIT)
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the ""Software""), to
# deal in the Software without restriction, including without limitation the
# rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
# sell copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED *AS IS*, WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.
#
# --------------------------------------------------------------------------
from __future__ import annotations
import logging
from collections.abc import Iterable
from typing import TypeVar, Generic, Optional, Any

from .pipeline import Pipeline
from ._base import PipelineClientBase
from ..transport import HttpTransport
from .policies import (
    ContentDecodePolicy,
    RetryPolicy,
    HeadersPolicy,
    UserAgentPolicy,
    ProxyPolicy,
    NetworkTraceLoggingPolicy,
)

HTTPResponseType = TypeVar("HTTPResponseType")
HTTPRequestType = TypeVar("HTTPRequestType")

_LOGGER = logging.getLogger(__name__)


class PipelineClient(PipelineClientBase, Generic[HTTPRequestType, HTTPResponseType]):
    """Service client core methods.

    Builds a Pipeline client.

    :param str endpoint: URL for the request.
    :keyword Pipeline pipeline: If omitted, a Pipeline object is created.
    :keyword list[HTTPPolicy] policies: If omitted, a set of standard policies is used.
    :keyword per_call_policies: If specified, the policies will be added into the policy list before RetryPolicy
    :paramtype per_call_policies: Union[HTTPPolicy, SansIOHTTPPolicy, list[HTTPPolicy], list[SansIOHTTPPolicy]]
    :keyword per_retry_policies: If specified, the policies will be added into the policy list after RetryPolicy
    :paramtype per_retry_policies: Union[HTTPPolicy, SansIOHTTPPolicy, list[HTTPPolicy], list[SansIOHTTPPolicy]]
    :keyword HttpTransport transport: If omitted, RequestsTransport is used for synchronous transport.

    :ivar pipeline: The Pipeline object associated with the client.
    :vartype pipeline: ~corehttp.runtime.pipeline.Pipeline or None

    .. admonition:: Example:

        .. literalinclude:: ../samples/sample_pipeline_client.py
            :start-after: [START build_pipeline_client]
            :end-before: [END build_pipeline_client]
            :language: python
            :dedent: 4
            :caption: Builds the pipeline client.
    """

    def __init__(
        self,
        endpoint: str,
        *,
        pipeline: Optional[Pipeline[HTTPRequestType, HTTPResponseType]] = None,
        **kwargs: Any,
    ) -> None:
        super().__init__(endpoint)
        self.pipeline = pipeline or self._build_pipeline(**kwargs)

    def __enter__(self) -> PipelineClient[HTTPRequestType, HTTPResponseType]:
        self.pipeline.__enter__()
        return self

    def __exit__(self, *exc_details: Any) -> None:
        self.pipeline.__exit__(*exc_details)

    def close(self) -> None:
        self.__exit__()

    def _build_pipeline(
        self,
        *,
        transport: Optional[HttpTransport[HTTPRequestType, HTTPResponseType]] = None,
        policies=None,
        per_call_policies=None,
        per_retry_policies=None,
        **kwargs,
    ) -> Pipeline[HTTPRequestType, HTTPResponseType]:
        per_call_policies = per_call_policies or []
        per_retry_policies = per_retry_policies or []

        if policies is None:
            policies = [
                kwargs.get("headers_policy") or HeadersPolicy(**kwargs),
                kwargs.get("user_agent_policy") or UserAgentPolicy(**kwargs),
                kwargs.get("proxy_policy") or ProxyPolicy(**kwargs),
                ContentDecodePolicy(**kwargs),
                kwargs.get("retry_policy") or RetryPolicy(**kwargs),
                kwargs.get("authentication_policy"),
                kwargs.get("logging_policy") or NetworkTraceLoggingPolicy(**kwargs),
            ]
        if isinstance(per_call_policies, Iterable):
            per_call_policies_list = list(per_call_policies)
        else:
            per_call_policies_list = [per_call_policies]
        per_call_policies_list.extend(policies)
        policies = per_call_policies_list

        if isinstance(per_retry_policies, Iterable):
            per_retry_policies_list = list(per_retry_policies)
        else:
            per_retry_policies_list = [per_retry_policies]
        if len(per_retry_policies_list) > 0:
            index_of_retry = -1
            for index, policy in enumerate(policies):
                if isinstance(policy, RetryPolicy):
                    index_of_retry = index
            if index_of_retry == -1:
                raise ValueError(
                    "Failed to add per_retry_policies; no RetryPolicy found in the supplied list of policies. "
                )
            policies_1 = policies[: index_of_retry + 1]
            policies_2 = policies[index_of_retry + 1 :]
            policies_1.extend(per_retry_policies_list)
            policies_1.extend(policies_2)
            policies = policies_1

        if transport is None:
            # Use private import for better typing, mypy and pyright don't like PEP562
            from ..transport.requests import RequestsTransport

            transport = RequestsTransport(**kwargs)

        return Pipeline(transport, policies)

    def send_request(self, request: HTTPRequestType, *, stream: bool = False, **kwargs: Any) -> HTTPResponseType:
        """Method that runs the network request through the client's chained policies.

        >>> from corehttp.rest import HttpRequest
        >>> request = HttpRequest('GET', 'http://www.example.com')
        <HttpRequest [GET], url: 'http://www.example.com'>
        >>> response = client.send_request(request)
        <HttpResponse: 200 OK>

        :param request: The network request you want to make. Required.
        :type request: ~corehttp.rest.HttpRequest
        :keyword bool stream: Whether the response payload will be streamed. Defaults to False.
        :return: The response of your network call. Does not do error handling on your response.
        :rtype: ~corehttp.rest.HttpResponse
        """
        pipeline_response = self.pipeline.run(request, stream=stream, **kwargs)
        return pipeline_response.http_response
